<?php

namespace EventPortal\Tests\Feature\Admin\User;

use Carbon\Carbon;
use EventPortal\Domain\Admin\Models\Admin;
use EventPortal\Domain\Component\Option;
use EventPortal\Domain\Team\Models\Team;
use EventPortal\Domain\User\Models\User;
use EventPortal\Tests\Support\Traits\SeedRolesAndPermissions;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Spatie\Snapshots\MatchesSnapshots;

class UserUpdateTest extends TestCase
{
    use RefreshDatabase, MatchesSnapshots, SeedRolesAndPermissions;

    /** @test **/
    public function the_edit_view_supplies_the_correct_data_to_the_frontend()
    {
        Carbon::setTestNow('2020-07-03 16:00:00');

        $adminA = factory(Admin::class)->state('super-admin')->create();
        $adminB = factory(Admin::class)->state('snapshot')->create();

        factory(Team::class)->state('snapshot')->create();

        $this->assertMatchesJsonSnapshot(
            $this->actingAs($adminA->user)
                ->get(route('admin.user.edit', $adminB->id))->props()
        );
    }

    /** @test */
    public function a_user_can_be_updated()
    {
        $admin = factory(Admin::class)
            ->state('super-admin')
            ->create([
                'user_id' => factory(User::class)->create([
                    'first_name' => 'Original First',
                    'last_name' => 'Original Last',
                ]),
            ]);

        $this
            ->actingAsSuperAdmin()
            ->followingRedirects()
            ->post(route('admin.user.update', $admin), [
                'first_name' => 'New First',
                'last_name' => 'New Last',
                'email' => 'person@example.com',
                'role_id' => Option::make('role_id')->value(1)->toArray(),
            ])
            ->assertOk();
        $this->assertEquals(route('admin.user.show', $admin), url()->current());

        $this->assertDatabaseHas('users', [
            'first_name' => 'New First',
            'last_name' => 'New Last',
        ]);
    }

    /** @test **/
    public function the_first_name_last_name_and_email_are_required()
    {
        $admin = factory(Admin::class)
            ->state('super-admin')
            ->create([
                'user_id' => factory(User::class)->create([
                    'first_name' => 'Original First',
                    'last_name' => 'Original Last',
                ]),
            ]);

        $this
            ->actingAsSuperAdmin()
            ->post(route('admin.user.update', $admin), [
                'first_name' => null,
                'last_name' => null,
            ])
            ->assertSessionHasErrors([
                'first_name',
                'last_name',
            ]);

        $admin->refresh();

        $this->assertEquals('Original First', $admin->user->first_name);
        $this->assertEquals('Original Last', $admin->user->last_name);
    }
}
